using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections.ObjectModel;
using System.Diagnostics;
using MtpsShowToc.Mtps;
using System.ComponentModel;
using System.Threading;
using System.Windows;
using System.Xml.Serialization;

namespace MtpsShowToc
{
    public class TocEntry
    {
        public XmlNode RawXml
        {
            get
            {
                return tocXml;
            }
        }
        private XmlNode tocXml;
        const string ns = "urn:mtpg-com:mtps/2004/1/toc";
        string title;
        string subtree, subtreeversion, subtreeLocale;
        ObservableCollection<TocEntry> children = new ObservableCollection<TocEntry>();
        bool needToFetchChildren;

        static XmlSerializer tocSerializer = new XmlSerializer(typeof(TocData));
        TocData td;
        public TocEntry(XmlNode tocNode)
        {
            tocXml = tocNode;

            XmlNamespaceManager nsm = new XmlNamespaceManager(tocNode.OwnerDocument.NameTable);
            nsm.AddNamespace("toc", ns);
            title = GetAttributeValue(tocNode, "Title");
            subtree = GetAttributeValue(tocNode, "SubTree");
            needToFetchChildren = subtree != null;
            if (needToFetchChildren)
            {
                Children.Add(new PlaceholderTocEntry(this));
            }

            subtreeversion = GetAttributeValue(tocNode, "SubTreeVersion");
            subtreeLocale = GetAttributeValue(tocNode, "SubTreeLocale");

            td = (TocData) tocSerializer.Deserialize(new XmlNodeReader(tocNode));

            /* If we were fetching the target documents described by the
             * ToC, we'd need these:
            string target = tocNode.Attributes["Target", ns].Value;
            string targetlocale = tocNode.Attributes["TargetLocale", ns].Value;
            string targetversion = tocNode.Attributes["TargetVersion", ns].Value;
            string isphantom = tocNode.Attributes["IsPhantom", ns].Value;
             */
        }

        /// <summary>
        /// Creates a root TocEntry
        /// </summary>
        public TocEntry()
        {
            title = "MSDN Library";
            subtree = "ms310241";
            subtreeLocale = "en-US";
            subtreeversion = "MSDN.10";
            needToFetchChildren = true;
            Children.Add(new PlaceholderTocEntry(this));


        }

        private TocEntry(string title)
        {
            this.title = title;
        }

        private string GetAttributeValue(XmlNode node, string name)
        {
            XmlAttribute attr = node.Attributes[name, ns];
            if (attr == null) return null;
            return attr.Value;
        }
        public string Title { get { return title; } }

        public virtual ObservableCollection<TocEntry> Children
        {
            get
            {
                return children;
            }
        }

        public class PlaceholderTocEntry : TocEntry
        {
            TocEntry parent;
            public PlaceholderTocEntry(TocEntry parent) : base("Fetching items...")
            {
                this.parent = parent;
            }

            public override ObservableCollection<TocEntry> Children
            {
                get
                {
                    Debug.WriteLine("Request for children of " + Title);
                    if (parent.needToFetchChildren) { parent.FetchChildren(); }
                    return null;
                }
            }
        }

        private void FetchChildren()
        {
            needToFetchChildren = false;

            SynchronizationContext syncContext = SynchronizationContext.Current;
            ThreadStart ts = delegate
            {
                getContentRequest req = new getContentRequest();
                req.contentIdentifier = subtree;
                req.locale = subtreeLocale;
                req.version = subtreeversion;

                requestedDocument[] docTypes = new requestedDocument[1];
                docTypes[0] = new requestedDocument();
                docTypes[0].type = documentTypes.primary;
                docTypes[0].selector = "Mtps.Toc";
                req.requestedDocuments = docTypes;

                try
                {
                    ContentServicePortTypeClient proxy = new ContentServicePortTypeClient("ContentService");
                    getContentResponse resp = proxy.GetContent(req);
                    Debug.WriteLine(resp.primaryDocuments.Length);

                    primary tocdoc = null;
                    foreach (primary pd in resp.primaryDocuments)
                    {
                        if (pd.primaryFormat == "Mtps.Toc")
                        {
                            tocdoc = pd;
                            break;
                        }
                    }
                    if (tocdoc != null)
                    {
                        if (tocdoc.Any != null)
                        {
                            XmlElement toc = tocdoc.Any;
                            XmlNamespaceManager nsm = new XmlNamespaceManager(toc.OwnerDocument.NameTable);
                            nsm.AddNamespace("toc", ns);
                            XmlNodeList nodes = toc.SelectNodes("/toc:Node", nsm);
                            syncContext.Post(delegate
                            {
                                // Remove dummy
                                Children.Clear();
                                foreach (XmlNode tocNode in nodes)
                                {
                                    TocEntry tocItem = new TocEntry(tocNode);
                                    Children.Add(tocItem);

                                    Debug.WriteLine(tocNode.OuterXml);
                                }
                            }, null);
                        }
                    }
                    else
                    {
                        //Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (EventHandler) delegate
                        syncContext.Post(delegate
                        {
                            // Remove dummy
                            Children.Clear();
                        //});
                        }, null);
                    }
                }
                catch (Exception x)
                {
                    Debug.WriteLine(x.ToString());
                    syncContext.Post(delegate
                    {
                        // Remove dummy
                        Children.Clear();

                        // We actually get errors quite a lot...
                        // In particular, the MTPS system quite often
                        // feeds out IDs that turn out not to exist, and
                        // cause an exception when you try to fetch them.

                        //MessageBox.Show(x.ToString(), "Error");
                    }, null);
                }
            };
            ts.BeginInvoke(delegate(IAsyncResult iar) { ts.EndInvoke(iar); }, null);
        }
    }
}