using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections.ObjectModel;
using System.Diagnostics;
using MtpsShowToc.Mtps;
using System.ComponentModel;
using System.Threading;
using System.Windows;

namespace MtpsShowToc
{
    public class TocEntrySync
    {
        const string ns = "urn:mtpg-com:mtps/2004/1/toc";
        string title;
        string subtree, subtreeversion, subtreeLocale;
        ObservableCollection<TocEntrySync> children = new ObservableCollection<TocEntrySync>();
        bool needToFetchChildren;

        public TocEntrySync(XmlNode tocNode)
        {

            XmlNamespaceManager nsm = new XmlNamespaceManager(tocNode.OwnerDocument.NameTable);
            nsm.AddNamespace("toc", ns);
            title = GetAttributeValue(tocNode, "Title");
            subtree = GetAttributeValue(tocNode, "SubTree");
            needToFetchChildren = subtree != null;

            subtreeversion = GetAttributeValue(tocNode, "SubTreeVersion");
            subtreeLocale = GetAttributeValue(tocNode, "SubTreeLocale");
        }

        /// <summary>
        /// Creates a root SyncTocEntry
        /// </summary>
        public TocEntrySync()
        {
            title = "MSDN Library";
            subtree = "ms310241";
            subtreeLocale = "en-US";
            subtreeversion = "MSDN.10";
            needToFetchChildren = true;
        }

        private TocEntrySync(string title)
        {
            this.title = title;
        }

        private string GetAttributeValue(XmlNode node, string name)
        {
            XmlAttribute attr = node.Attributes[name, ns];
            if (attr == null) return null;
            return attr.Value;
        }
        public string Title { get { return title; } }

        public virtual ObservableCollection<TocEntrySync> Children
        {
            get
            {
                if (needToFetchChildren) { FetchChildren(); }
                return children;
            }
        }

        private void FetchChildren()
        {
            needToFetchChildren = false;

            getContentRequest req = new getContentRequest();
            req.contentIdentifier = subtree;
            req.locale = subtreeLocale;
            req.version = subtreeversion;

            requestedDocument[] docTypes = new requestedDocument[1];
            docTypes[0] = new requestedDocument();
            docTypes[0].type = documentTypes.primary;
            docTypes[0].selector = "Mtps.Toc";
            req.requestedDocuments = docTypes;

            try
            {
                ContentServicePortTypeClient proxy = new ContentServicePortTypeClient("ContentService");
                getContentResponse resp = proxy.GetContent(req);
                Debug.WriteLine(resp.primaryDocuments.Length);

                if (resp.primaryDocuments.Length > 0 && resp.primaryDocuments[0].primaryFormat == "Mtps.Toc")
                {
                    primary tocdoc = resp.primaryDocuments[0];
                    if (tocdoc.Any != null)
                    {
                        XmlElement toc = tocdoc.Any;
                        XmlNamespaceManager nsm = new XmlNamespaceManager(toc.OwnerDocument.NameTable);
                        nsm.AddNamespace("toc", ns);
                        XmlNodeList nodes = toc.SelectNodes("/toc:Node", nsm);
                        foreach (XmlNode tocNode in nodes)
                        {
                            TocEntrySync tocItem = new TocEntrySync(tocNode);
                            Children.Add(tocItem);

                            Debug.WriteLine(tocNode.OuterXml);
                        }
                    }
                }
            }
            catch (Exception x)
            {
                Debug.WriteLine(x.ToString());
            }
        }
    }
}